﻿using log4net;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Serialization;
using VehicleStatsBL.Interface;

namespace VehicleStatsBL.Impl
{
    public class BinaryResultsStreamer : IResultsStreamer
    {
        private ILog _log;
        public BinaryResultsStreamer(ILog log)
        {
            _log = log;
        }
        public IExtractionResults Read(string path)
        {
            using (FileStream fs = new FileStream(path, FileMode.Open))
            {
                BinaryFormatter formatter = new BinaryFormatter();
                try
                {
                    return (IExtractionResults) formatter.Deserialize(fs);
                }
                catch (SerializationException e)
                {
                    _log.Error("Failed to deserialize. Reason: " + e.Message);
                    throw;
                }
            }
        }

        public void Write(IExtractionResults extractionResults, string path)
        {
            using (FileStream fs = new FileStream(path, FileMode.Create))
            {
                BinaryFormatter formatter = new BinaryFormatter();
                try
                {
                    formatter.Serialize(fs, extractionResults);
                    _log.Debug("Results streamed to " + path);
                }
                catch (SerializationException e)
                {
                    _log.Error("Failed to serialize. Reason: " + e.Message);
                    throw;
                }
            }
        }
    }
}
