﻿using HtmlAgilityPack;
using log4net;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using VehicleStatsBL.Interface;

namespace VehicleStatsBL.Impl
{
    public class ExtractionEngine
    {
          
        private object _lockObject = new object();
        private IHtmlWebWrapper _htmlWebWrapper;
        private ILog _log;
        private IPageScraper _pageScraper;
        
        public ExtractionEngine(IHtmlWebWrapper htmlWebWrapper, ILog log, IPageScraper pageScraper)
        {
            _log = log;
 
            _htmlWebWrapper = htmlWebWrapper;
            _pageScraper = pageScraper;
            
        }

        public void Extract(IExtractionArguments args, IExtractionResults extractionResults)
        {
            _log.Debug("Extraction starting");
            extractionResults.Start();

            var firstPageUrl = _pageScraper.GetFirstPageUrl(args);
            HtmlDocument firstPage = _htmlWebWrapper.Load(firstPageUrl.OriginalString);

            extractionResults.Vehicles.AddRange(_pageScraper.Scrape(args, firstPage));

            List<string> remainingUrls = _pageScraper.GetRemainingUrls(firstPage);

            Parallel.ForEach(remainingUrls, pagedUrl =>
            {
                var resultsPage = _htmlWebWrapper.Load(pagedUrl);

                lock (_lockObject)
                {
                    extractionResults.Vehicles.AddRange(_pageScraper.Scrape(args, resultsPage));
                }

            });

            extractionResults.Stop();
        }
    }
}
