﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using VehicleStatsBL.Interface;

namespace VehicleStatsBL.Impl
{
    [Serializable]
    public class ExtractionResults : IExtractionResults
    {
        private List<IVehicle> _vehicles;
        
        [NonSerialized]
        private Stopwatch _stopwatch;
        private IStatisticsFactory _statisticsFactory;

        public ExtractionResults(IStatisticsFactory statisticsFactory)
        {
            _stopwatch = new Stopwatch();
            _vehicles = new List<IVehicle>();
            _statisticsFactory = statisticsFactory;
        }

        public List<IVehicle> Vehicles
        {
            get { return _vehicles; }
        }

        public TimeSpan OperationDuration
        {
            get { return _stopwatch.Elapsed; }
        }

        public void Stop()
        {
            if (_stopwatch != null)
                _stopwatch.Stop();
        }

        public void Start()
        {
            _stopwatch.Start();
        }

        public int LowestYear
        {
            get
            { return _vehicles.OrderBy(p => p.Year).FirstOrDefault().Year; }
        }

        public int HighestYear
        {
            get
            { return _vehicles.OrderByDescending(p => p.Year).FirstOrDefault().Year; }
        }

        public IStatistics SampleStatistics
        {
            get {
                return _statisticsFactory.GetStatistics(_vehicles);
            }
        }

        public IStatistics GetStatisticsByYearRange(int from, int to)
        {
            return _statisticsFactory.GetStatistics(_vehicles.Where(p => (p.Year >= from) && (p.Year <= to)));
        }


        public IStatistics GetStatisticsByTitleCriteria(string searchString)
        {
            return _statisticsFactory.GetStatistics(_vehicles.Where(p => p.Title.ToUpper().Contains(searchString.ToUpper())));
        }
    }
}
