﻿using log4net;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Rhino.Mocks;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using VehicleStatsBL.Impl;
using VehicleStatsBL.Interface;

namespace VehicleStatsBLTests
{
     [TestClass]
    public class BinaryResultsStreamerTest
    {
         private const string TestPath = "TestData";
         private const string TestDataFile = @"volvo[s60] 2000_2013 (20131111).dat";

         private string ResolvedTestPath;
         private string ResolvedTestFile;
         private string DummyOutputFile;

         private ILog _log;
         private BinaryResultsStreamer _streamer;
         private StatisticsFactory _statisticsFactory;
         private ExtractionResults _results;
         private Vehicle _testVehicle;

         [TestInitialize]
         public void Init()
         {
             _log = Rhino.Mocks.MockRepository.GenerateMock<ILog>();
             _streamer = new BinaryResultsStreamer(_log);
             _statisticsFactory = new StatisticsFactory();
             _results = new ExtractionResults(_statisticsFactory);
             _testVehicle = new Vehicle() { Make = "bmw", Model = "3-series", Milage = "100", Price = 100000, Title = "BMW M3", Year = 2013 };
             _results.Vehicles.Add(_testVehicle);
          

             ResolvedTestPath = Path.GetFullPath(TestPath);
             ResolvedTestFile = Path.GetFullPath(string.Format("{0}\\{1}", TestPath, TestDataFile));
             DummyOutputFile = string.Format("{0}\\{1}", ResolvedTestPath, "dummy.dat");
         }

         [TestCleanup]
         public void Cleanup()
         {
             if (File.Exists(DummyOutputFile))
                 File.Delete(DummyOutputFile);
         }

         [TestMethod]
         public void test_that_you_can_deserialise()
         {
             var extractionResults = _streamer.Read(ResolvedTestFile);

             Assert.IsNotNull(extractionResults);
         }

         [TestMethod]
         public void test_that_you_can_serialise()
         {
             _streamer.Write(_results, DummyOutputFile);

             Assert.IsTrue(File.Exists(DummyOutputFile));
         }

         [TestMethod]
         public void test_that_you_can_serialise_and_deserialse()
         {
             _streamer.Write(_results, DummyOutputFile);
             var extractionResults = _streamer.Read(DummyOutputFile);

             Assert.IsNotNull(extractionResults);
         }


         [TestMethod]
         [ExpectedException(typeof(FileNotFoundException))]
         public void test_that_it_handles_files_that_do_not_exist()
         {
             var extractionResults = _streamer.Read(Path.GetFullPath(TestDataFile + "rubbish"));
         }
    }
}
